@@ -1,7 +1,9 @@
 /**
  * \file timing.h
  *
- *  Copyright (C) 2006-2010, Brainspark B.V.
+ * \brief Portable interface to the CPU cycle counter
+ *
+ *  Copyright (C) 2006-2013, Brainspark B.V.
  *
  *  This file is part of PolarSSL (http://www.polarssl.org)
  *  Lead Maintainer: Paul Bakker <polarssl_maintainer at polarssl.org>
@@ -25,6 +27,10 @@
 #ifndef POLARSSL_TIMING_H
 #define POLARSSL_TIMING_H
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /**
  * \brief          timer structure
  */
@@ -33,11 +39,7 @@ struct hr_time
     unsigned char opaque[32];
 };
 
-#ifdef __cplusplus
-extern "C" {
-#endif
-
-extern int alarmed;
+extern volatile int alarmed;
 
 /**
  * \brief          Return the CPU cycle counter value
@@ -62,7 +64,7 @@ void set_alarm( int seconds );
 /**
  * \brief          Sleep for a certain amount of time
  *
- * \param          Delay in milliseconds
+ * \param milliseconds  delay in milliseconds
  */
 void m_sleep( int milliseconds );
 
