@@ -23,14 +23,10 @@
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-#ifndef _CRT_SECURE_NO_DEPRECATE
-#define _CRT_SECURE_NO_DEPRECATE 1
-#endif
+#include "polarssl/config.h"
 
 #include <stdio.h>
 
-#include "polarssl/config.h"
-
 #include "polarssl/md5.h"
 
 #if !defined(POLARSSL_MD5_C)
